@@ -63,6 +63,7 @@ static int act_keyctl_reject(int argc, char *argv[]);
 static int act_keyctl_reap(int argc, char *argv[]);
 static int act_keyctl_purge(int argc, char *argv[]);
 static int act_keyctl_invalidate(int argc, char *argv[]);
+static int act_keyctl_get_persistent(int argc, char *argv[]);
 
 const struct command commands[] = {
 	{ act_keyctl___version,	"--version",	"" },
@@ -73,6 +74,7 @@ const struct command commands[] = {
 	{ act_keyctl_describe,	"describe",	"<keyring>" },
 	{ act_keyctl_instantiate, "instantiate","<key> <data> <keyring>" },
 	{ act_keyctl_invalidate,"invalidate",	"<key>" },
+	{ act_keyctl_get_persistent, "get_persistent", "<keyring> [<uid>]" },
 	{ act_keyctl_link,	"link",		"<key> <keyring>" },
 	{ act_keyctl_list,	"list",		"<keyring>" },
 	{ act_keyctl_negate,	"negate",	"<key> <timeout> <keyring>" },
@@ -111,7 +113,7 @@ const struct command commands[] = {
 static int dump_key_tree(key_serial_t keyring, const char *name, int hex_key_IDs);
 static void format(void) __attribute__((noreturn));
 static void error(const char *msg) __attribute__((noreturn));
-static key_serial_t get_key_id(const char *arg);
+static key_serial_t get_key_id(char *arg);
 
 static uid_t myuid;
 static gid_t mygid, *mygroups;
@@ -1573,11 +1575,43 @@ static int act_keyctl_invalidate(int argc, char *argv[])
 	return 0;
 }
 
+/*****************************************************************************/
+/*
+ * Get the per-UID persistent keyring
+ */
+static int act_keyctl_get_persistent(int argc, char *argv[])
+{
+	key_serial_t dest, ret;
+	uid_t uid = -1;
+	char *q;
+
+	if (argc != 2 && argc != 3)
+		format();
+
+	dest = get_key_id(argv[1]);
+
+	if (argc > 2) {
+		uid = strtoul(argv[2], &q, 0);
+		if (*q) {
+			fprintf(stderr, "Unparsable uid: '%s'\n", argv[2]);
+			exit(2);
+		}
+	}
+
+	ret = keyctl_get_persistent(uid, dest);
+	if (ret < 0)
+		error("keyctl_get_persistent");
+
+	/* print the resulting key ID */
+	printf("%d\n", ret);
+	return 0;
+}
+
 /*****************************************************************************/
 /*
  * parse a key identifier
  */
-static key_serial_t get_key_id(const char *arg)
+static key_serial_t get_key_id(char *arg)
 {
 	key_serial_t id;
 	char *end;
@@ -1596,6 +1630,36 @@ static key_serial_t get_key_id(const char *arg)
 		exit(2);
 	}
 
+	/* handle a lookup-by-name request "%<type>:<desc>", eg: "%keyring:_ses" */
+	if (arg[0] == '%') {
+		char *type;
+
+		arg++;
+		if (!*arg)
+			goto incorrect_key_by_name_spec;
+
+		if (*arg == ':') {
+			type = "keyring";
+			arg++;
+		} else {
+			type = arg;
+			arg = strchr(arg, ':');
+			if (!arg)
+				goto incorrect_key_by_name_spec;
+			*(arg++) = '\0';
+		}
+
+		if (!*arg)
+			goto incorrect_key_by_name_spec;
+
+		id = find_key_by_type_and_desc(type, arg, 0);
+		if (id == -1) {
+			fprintf(stderr, "Can't find '%s:%s'\n", type, arg);
+			exit(1);
+		}
+		return id;
+	}
+
 	/* handle a numeric key ID */
 	id = strtoul(arg, &end, 0);
 	if (*end) {
@@ -1605,6 +1669,10 @@ static key_serial_t get_key_id(const char *arg)
 
 	return id;
 
+incorrect_key_by_name_spec:
+	fprintf(stderr, "Incorrect key-by-name spec\n");
+	exit(2);
+
 } /* end get_key_id() */
 
 /*****************************************************************************/
